# HABSKYX -- Divide extracted multispec spectrum file by scaled sky absorption spectra
#          saving original file in Noskyab/
# By Doug Mink, SAO Telescope Data Center
# May 22, 2007

procedure habskyx (image)

string	image = ""	{prompt="Input multispec image to correct"}
string	skyfile="hectospec$lib/skyabs0.1.ms.fits"	{prompt="Sky absorption spectrum normalized to continuum=1"}
real	scale = 0	{prompt="Scale factor for sky absorption (0=none)"}
bool	deltemp = no	{prompt="Delete temporary files (y or n)"}
bool	saveold = yes	{prompt="Save unprocessed file in Noskyab/ (y or n)"}
bool	verbose = yes	{prompt="Log progress (y or n)"}
bool	debug = no	{prompt="Log progress (y or n)"}

begin
	bool vb, db
	file newfile, imin, imsky, imoutf, ssky1, ssky2, ssky
	int lroot
	real ss
	string imout, history, ap, keypath
	string bands	# List of input bands or lines/bands
	int npix, nap

	bands=""
	imin = image
	vb = verbose
	db = debug
	imsky = skyfile
	db = debug
	ss = scale

	if (!defpac ("wcstools")) {
	    wcstools (motd=no)
	    }

	if (imin == "") {
	    print ("HABSKY1:  Divide extracted multispec spectrum file by sky absorption spectrum")
	    print ("Usage: habsky1 <input image>")
	    return
	    }

# Make sure input is a FITS multispec file
	smatch (imin, ".ms")
	if (!smatch.found) {
	    smatch (imin, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imin = substr (imin, 1, lroot) // ".ms.fits"
		}
	    else {
		imin = imin // ".ms.fits"
		}
	    }
	else {
	    smatch (imin, ".fits")
	    if (!smatch.found) {
		imin = imin // ".fits"
		}
	    }
	if (!access (imin)) {
	    print ("HABSKY1: ", imin," does not exist")
	    return
	    }

# Do not remove sky absorption twice
	imgetp.default = "0"
	imgetp (imin, "HABSKY")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("HABSKY: ", imin, " has already been processed")
		print ("HABSKY = ", imgetp.value)
		}
	    return
	    }

# Copy original file to Noskyab directory
	if (saveold) {
	    if (!access ("Noskyab/")) {
		mkdir ("Noskyab")
		}
	    newfile = "Noskyab/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    imcopy.verbose = db
	    imcopy (imin, "Noskyab")
	    if (vb) {
		print ("HABSKY1: Original ", imin, " => Noskyab/")
		}
	    }


# Get number of apertures and pixels from header of input multispec image
	imgetp.verbose = db
	imgetp.default = "0"
	imgetp (imin, "i_naxis1")
	npix = int (imgetp.value)
	imgetp (imin, "i_naxis2")
	nap = int (imgetp.value)
	if (vb) {
	    print ("HABSKY1: ", nap," apertures of ",npix," pixels each")
	    }

# Set up sarith parameters
	sarith.w1 = INDEF
	sarith.w2 = INDEF
	sarith.apertures = "1-" // nap
	sarith.bands = bands
	sarith.beams = ""
	sarith.apmodulus = 0
	sarith.reverse = no
	sarith.ignoreaps = yes
	sarith.format = "multispec"
	sarith.renumber = no
	sarith.offset = 0
	sarith.clobber = yes
	sarith.merge = no
	sarith.rebin = yes
	sarith.errval = 0.
	sarith.verbose = db
	imout = mktemp ("abskyout") // ".ms.fits"
	imoutf = mktemp ("abskyoutf") // ".ms.fits"
	if (vb) {
	    print ("Scaling sky by ", ss)
	    }

# Rescale absorption lines and divide them out
	if (ss > 0.0) {
	    ssky1 = mktemp ("abskyscale") // ".ms.fits"
	    ssky2 = mktemp ("abskyscale") // ".ms.fits"
	    ssky = mktemp ("abskyscale") // ".ms.fits"
	    if (db) {
		print ("Subtracting 1 from sky")
		}
	    sarith (imsky, "-", "1.0", ssky1)
	    if (db) {
		print ("Multiplying sky by ", ss)
		}
	    sarith (ssky1, "*", ss, ssky2)
	    if (db) {
		print ("Adding 1 back into sky")
		}
	    sarith (ssky2, "+", "1.0", ssky)
	    if (db) {
		print ("Dividing object spectra by sky")
		}
	    sarith (imin, "/", ssky, imout)
	    }

# Divide out unscaled absorption spectra
	else {
	    sarith (imin, "/", imsky, imout)
	    }

# Fix APID keywords
	imgetp.default = "0"
	imgetp (imin, "APID1")
	if (imgetp.value != "0") {
	    imcopy (imout, imoutf)
	    print ("Sky removed in ",imout)
#	    keypath = "@" // osfn("hectospec$lib/") // "apnumkeys"
#	    print ("delhead ",imoutf," ",keypath)
#	    dhead (imoutf,keypath)
	    keypath = "@" // osfn("hectospec$lib/") // "apidkeys"
	    print ("cphead ",imin," ",imoutf," ",keypath)
	    chead (imin,imoutf,keypath)
	    }
	else {
	    imoutf = imout
	    }

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " " // imsky // " divided out"
	hedit (imoutf, "HABSKY", history, add+, verify-, show-)
	if (vb) {
	    print ("HABSKY: ",history)
	    }

# Rename output file to same as input file (which is saved if saveold=yes)
	imdelete (imin)
	imcopy (imoutf, imin)

	if (deltemp) {
	    imdelete (imout)
	    imdelete (imoutf)
	    if (ss > 0.0) {
		imdelete (ssky1)
		imdelete (ssky2)
		imdelete (ssky)
		}
	    }
end

# Nov  8 2006	New program based on habsky1.cl

# May 22 2007	Only check for wcstools package, not chead task in it
