# HNCOSMIC - Remove cosmic rays from a list of images
# May 15, 2007
# By Doug Mink, SAO Telescope Data Center
# CL and SSP implementation of Nelson Caldwell's Perl scripts

procedure hncosmic (images)

string	images=""	{prompt="Images to combine"}
int	threshold=20	{prompt="Threshold for cosmic ray difference"}
bool	recompute=no	{prompt="Recompute median if already there (y or n)"}
bool	deltemp=no	{prompt="Delete intermediate image files (y or n)"}
bool    saveold=yes     {prompt="Save unprocessed image files to Nocosmic/ (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	medfile, limfile, templist, imin1, imin2, imbp1, imbp2, imr
int	ifile, nfiles, lroot, pass
string	tchar, imagelist, history
string	outroot, fileroot, imroot
file	listfile, imfile
real	medmed, high
bool	dellist, vb, recomp

	vb = verbose
	dellist = deltemp
	imagelist = images
	recomp = recompute
	high = threshold

# Make directory in which to save preampfixed multi-extension files
	if (!access ("Uncosmic")) {
	    mkdir ("Uncosmic")
	    }

# Load needed packages if necessary
	if (!defpac ("imred")) {
	    imred
	    }
	if (!defpac ("proto")) {
	    proto
	    }

#  Make temporary list of input files
	templist = mktemp ("hncosmic") // ".list"
	files (imagelist, >templist)

# Count files, get spectrum dimension, and initialize output root name
	nfiles = 0
	list = templist
	while (fscan (list,imin1) != EOF) {
	    nfiles = nfiles + 1
	    if (nfiles == 1) {
		smatch (imin1, ".fits")
		if (smatch.found) {
		    lroot = smatch.start - 6
		    fileroot = substr (imin1, 1, lroot+5)
		    outroot = substr (imin1, 1, lroot)
		    }
		else {
		    fileroot = imin1
		    lroot = strlen (imin1) - 5
		    outroot = substr (imin1, 1, lroot)
		    }
		}
	    }
	if (nfiles == 0) {
	    print ("No files to process")
	    return
	    }
	else if (vb) {
	    print (nfiles," files to process")
	    }

#  Make variance of median of input images
	medfile = outroot // "_med.fits"
	if (!access (medfile) || recompute) {
	    hcosmed (imagelist, medfile=medfile, nfiles=nfiles, verbose=vb)
	    }
	limfile = outroot // "_lim.fits"
	if (!access (limfile) || recompute) {
	    hcoslim (medfile, limfile, verbose=vb)
	    }
	hcosmic.limfile = limfile
	hcosmic.value = high
	hcosmic.verbose = vb
	hcosmic.proc2 = yes
	imreplace.lower = 2.0
	imreplace.upper = 4.0
	imreplace.radius = 1.5
	fixpix.linterp = "INDEF"
	fixpix.cinterp = "INDEF"
	fixpix.verbose = vb
	fixpix.pixels = no
	imin1 = ""
	pass = 0

# Compare images two at a time to remove cosmic rays
	list = templist
	while (fscan (list,imin1) != EOF) {

	# Find root of file name
	    smatch (imin1, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imin1, 1, lroot)
		}
	    else {
		imroot = imin1
		}
	    imbp1 = imroot // "_bp.fits"

	#  Save original file to Uncosmic/
	    if (saveold) {
		imr = "Uncosmic/" // imin1
		if (!access (imr))
		    copy (imin1, imr)
		}

	    if (imin2 != "") {
		if (pass == 1) {

		# Find the cosmic rays in these two images
		    if (vb) {
			print ("Marking cosmic rays in ",imin1," and ",imin2)
			print ("to bad pixel files ",imbp1," and ",imbp2)
			}
		    if (!access (imbp1) && !access (imbp2)) {
			hcosmic (imin1,imin2,imbp1,imbp2)
			imreplace (imbp1,3)
			hedit (imbp1, "IMREP", "Cosmic rays expanded by 1.5 pixels")
			imreplace (imbp2,3)
			hedit (imbp2, "IMREP", "Cosmic rays expanded by 1.5 pixels")
			}
		    else {
			print ("Bad pixel files ",imbp1," and ",imbp2," already exist.")
			}

		# Since we want to run FIXPIX again, delete FIXPIX keywords
		    imgetp (imin1, "FIXPIX")
		    hedit (imin1, "FIXPIX0", imgetp.value, add+)
		    hedit (imin1, "FIXPIX", del+)
		    imgetp (imin2, "FIXPIX")
		    hedit (imin2, "FIXPIX0", imgetp.value, add+)
		    hedit (imin2, "FIXPIX", del+)

		# Interpolate along columns across bad pixels
		    if (vb) {
			print ("Removing cosmic rays in ",imin1," and ",imin2)
			}
		    fixpix (imin1, imbp1)
		    fixpix (imin2, imbp2)

		# Make history line including time for preamp correction
		    gettime ()
		    history = gettime.date // " " // gettime.time
		    history = history // " cosmic rays removed"
		    hedit (imin1, "HNCOSMIC", history, add+, verify-)
		    hedit (imin2, "HNCOSMIC", history, add+, verify-)
		    hcosmic.proc2 = no
		    }

	    # After doing the first pair, operate only on the last-read image
		else {

		# Find the cosmic rays in the latest image
		    if (vb) {
			print ("Marking cosmic rays in ",imin1)
			}
		    if (!access (imbp1)) {
			hcosmic (imin1,imin2,imbp1,imbp2)
			imreplace (imbp1,3)
			hedit (imbp1, "IMREP", "Cosmic rays expanded by 1.5 pixels")
			}
		    else {
			print ("Bad pixel file ",imbp1," already exists.")
			}

		# Since we want to run CCDPROC again, delete FIXPIX keyword
		    imgetp (imin1, "FIXPIX")
		    hedit (imin1, "FIXPIX0", imgetp.value, add+)
		    hedit (imin1, "FIXPIX", del+)

		# Interpolate along columns across bad pixels
		    if (vb) {
			print ("Removing cosmic rays in ",imin1)
			}
		    fixpix (imin1, imbp1)

		# Make history line including time for preamp correction
		    gettime ()
		    history = gettime.date // " " // gettime.time
		    history = history // " cosmic rays removed"
		    hedit (imin1, "HNCOSMIC", history, add+, verify-)
		    }
		}
	    imin2 = imin1
	    imbp2 = imbp1
	    pass = pass + 1
	    }
	if (dellist) {
	    delete (templist)
	    }
end
# May 15 2007	New program based on Nelson Caldwell's Perl scripts
