# HREFIT -- Fit sky in band 3 to object + sky in band 2 and write new
#           multispec file in outdir
# By Doug Mink, SAO Telescope Data Center
# November 20, 2006

procedure hrefit (images)

string	images = ""	{prompt="Input multispec images to correct"}
file	outdir = "skysub/"	{prompt="Output directory"}
real	fitmax = 1.2	{prompt="Maximum acceptable fit coefficient"}
bool	plotres = no	{prompt="Plot sky-subtracted spectrum (y or n)"}
bool	deltemp = yes	{prompt="Delete temporary list of images (y or n)"}
bool	verbose = yes	{prompt="Log progress (y or n)"}
bool	debug = no	{prompt="Log progress (y or n)"}

begin
	bool vb, db
	file newfile, templist, imin
	string imout, history, ims, tchar

	ims = images
	vb = verbose
	db = debug

	if (ims == "") {
	    print ("HREFIT:  Refit and remove sky from multispec spectrum")
	    print ("Usage: hrefit <input images>")
	    return
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
        if (tchar == '@') {
	    templist = substr (ims,2,100)
	    }
	else {
	    templist = mktemp ("abskylist")
	    files (ims, >templist)
	    }
        list = templist
        while (fscan (list,imin) != EOF) {

# Make sure input is a FITS file
	    smatch (imin, ".ms")
	    if (!smatch.found) {
		smatch (imin, ".fit")
		if (!smatch.found) {
		    imin = imin // ".fits"
		    if (!access (imin)) {
			print ("HREFIT: ", imin," is not a FITS file")
			next
			}
		    }
		}

	    hskyfit1.objfile = imin
	    hskyfit1.skyfile = imin
	    hskyfit1.outfile = imin
	    hskyfit1.objap = 2
	    hskyfit1.objband = 1
	    hskyfit1.objdir = ""
	    hskyfit1.fitmax = fitmax
	    hskyfit1.plotres = plotres
	    hskyfit1.skyap = 3
	    hskyfit1.skyband = 1
	    hskyfit1.skydir = ""
	    hskyfit1.outdir = outdir
	    hskyfit1.verbose = verbose
	    hskyfit1.debug = debug

	    hskyfit1 (imin, imin, imin)

# Add history line including time
	    imout = outdir // imin
	    gettime ()
	    history = gettime.date // " " // gettime.time // " sky divided out"
	    hedit (imout, "HREFIT", history, add+, verify-, show-)
	    if (vb) {
		print ("HREFIT: ",history)
		}
	    }

	if (tchar != '@' && deltemp) {
	    delete (templist)
	    }
end

# Nov 20 2006	New program
