# File hectospec/Scripts/hskyfilt.cl
# By Doug Mink, after Bill Wyatt
# September 13, 2007

procedure hskyfilt (inlist)

file	inlist="sky.db"	{prompt="File of file names to test"}

file	outlist=""	{prompt="Name of filtered output file"}
file	destdir=""	{prompt="Name of directory for sky-subtracted output"}
real	skycert=4.0	{prompt="Max R value for certified sky"}
real	skystdlim=0.2	{prompt="Max std of difference from average sky"}
string	abtemps="@hectospec$hskyabtemps"	{prompt="Absorption line templates for sky testing"}
string	emtemps="femtemp97"	{prompt="Emission line templates for sky testing"}
int	nsky=0		{prompt="Number of usable sky spectra (returned)"}
bool	deltemp=yes	{prompt="Delete temporary files (y or n)"}
bool	verbose=yes	{prompt="Note progress"}
bool	debug=no	{prompt="Note progress in detail"}

begin
	bool vb, dt, db
	string vstring, comlist,mean,stddev,filelist, args
	string tline1, tline2,ts1,ts2,ts3,ts4,ts5
	file skylist, skysub, skyavg, inflist, imsky, skydb, ppath, ddir
	int insky, nsky0, nskyp, ap, fib
	real rab, rem, fibx, fiby, fibr

	if (!deftask ("pxcsao"))
	    rvsao (motd=no)
	dt = deltemp
	vb = verbose
	db = debug
	skylist = inlist
	if (!access (skylist)) {
	    print ("HSKYFILT: Cannot read list file ",skylist)
	    return
	    }
	nsky = 0
	ddir = destdir
	fixdir (ddir)
	ddir = fixdir.newdir

# If file has already been filtered, return
	fileroot (skylist)
	outlist = fileroot.rootname // "f.db"
	if (access (outlist)) {
	    wc (outlist) | scan (nsky)
	    if (vb) {
		print ("HSKYFILT: ",nsky," skies from existing file ",outlist)
		}
	    return
	    }

# Make temporary files for mean and sky-divided spectra
	skysub = mktemp ("tempsub") // ".fits"
	skyavg = mktemp ("tempav") // ".fits"
	filelist = mktemp ("tempfilelist")
	comlist = "@" // filelist
	inflist = mktemp ("tempinlist")
	copy (skylist, inflist)
	tline1 = "filename   	ap 	fiber	fiber_x	fiber_y	fiber_r"
	tline2 = "-----------	---	-----	-------	-------	-------"

# Compute RMS of individual skys against the sum of them; test for rejection
	imcombine.rejmask=""
	imcombine.combine="average"
	imcombine.reject="avsigclip"
	imcombine.outtype="real"
	imcombine.scale="none"
	imcombine.zero="none"
	imcombine.weight="!EXPTIME"
	imcombine.mclip=yes
	imcombine.lsigma=3.
	imcombine.hsigma=3.
	imcombine.sigscale=0.1
	imcombine.logfile=""
	if (vb)
	    print ("HSKYFILT: Comparing standard deviation of sky / (mean sky)")
	for (i = 0; i < 3; i = i + 1) {
	    if (i > 0 && vb) {
		print ("HSKYFILT Pass ",i,": standard deviation of sky / (mean sky)")
		}
	    if (access (skyavg))
		delete (skyavg)
	    if (access (filelist))
		delete (filelist)
	    list = inflist
	    if (fscan (list, ts1,ts2,ts3,ts4,ts5) != EOF) {
        	if (fscan (list, ts1,ts2,ts3,ts4,ts5) != EOF) {
		    }
        	}
	    insky = 0
	    while (fscan (list, imsky, ap, fib, fibx, fiby, fibr) != EOF) {
		insky = insky + 1
		    if (db)
			print (insky,": ", imsky)
		files (imsky, >>filelist)
		}
	    imcombine (comlist, skyavg)
	    if (skystdlim <= 0.0)
		break
	    if (access (outlist))
		delete (outlist)
	    nsky = 0
	    nsky0 = 0
	    nskyp = 0
	    list = inflist
	    if (fscan (list, ts1,ts2,ts3,ts4,ts5) != EOF) {
        	if (fscan (list, ts1,ts2,ts3,ts4,ts5) != EOF) {
		    }
        	}
	    while (fscan (list, imsky, ap, fib, fibx, fiby, fibr) != EOF) {
		if (!access (imsky)) {
		    print ("HSKYFILT: Cannot read sky spectrum ",imsky)
		    next
		    }
		imarith (imsky, "/", skyavg, skysub)
		istat (skysub,fields="image,mean,stddev",lower=INDEF,upper=INDEF,nclip=0,print=db,format=db)
		imdelete (skysub)
		round (istat.mean, "%.4f")
		mean = round.value
		round (istat.stddev, "%.4f")
		stddev = round.value
		if (vb)
		    print (imsky," ",mean," ",stddev)
		hedit (imsky, "SKYMEAN", mean, add+, verify-, show-)
		hedit (imsky, "SKYSTDEV", stddev, add+, verify-, show-)
		vstring = insky
		hedit (imsky, "CHIPSKY", vstring, add+, verify-, show-)
		if (ap < 151)
		    hedit (imsky, "CHIPNUM", "1", add+, verify-, show-)
		else
		    hedit (imsky, "CHIPNUM", "2", add+, verify-, show-)
		nskyp = nskyp + 1
		if (i == 0)
		    nsky0 = nsky0 + 1
		if (istat.stddev > skystdlim) {
		    vstring = "std_dev " // stddev // " > " // skystdlim
		    hedit (imsky, "SKYREJ", vstring, add+, verify-, show-)
		    print (imsky," rejected: ",vstring)
		    }
		else {
		    nsky = nsky + 1
		    if (nsky == 1) {
			printf ("%s\n%s\n", tline1, tline2, > outlist)
			}
		    printf ("%s	%3d	%3d	%7.2f	%7.2f	%7.2f\n",imsky, ap,fib,fibx,fiby, fibr, >>outlist)
		    }
		}
	    if (nsky < 1) {
		print ("HSKYFILT: No skies passed stdev test")
		if (dt) {
		    if (access (skysub))
			delete (skysub)
		    delete (skyavg)
		    }
		return
		}
	    else if (vb) {
		if (nsky == nsky0)
		    print ("HSKYFILT: All ",nsky," skies passed the stdev test")
		else
		    print ("HSKYFILT: ",nsky,"/",nsky0," skies passed the stdev test")
		}
	    if (nsky == nskyp)
		break
	    nskyp = nsky
	    delete (inflist)
	    rename (outlist, inflist)
	    }

# Initialize PXCSAO
	pxcsao.tempdir = "rvsao$Templates/"
	if (db) {
	    pxcsao.logfiles = "xcsao.log,STDOUT"
	    }
	else {
	    pxcsao.logfiles = "xcsao.log"
	    }

# Initialize CONTPARS (continuum fit for PXCSAO)
	contpars.c_interactive = no
	contpars.c_sample = "*"
	contpars.c_function = "spline3"
	contpars.naverage = 1
	contpars.order = 15
	contpars.s_low_reject = 2.
	contpars.s_high_reject = 2.
	contpars.t_low_reject = 2.
	contpars.t_high_reject = 2.
	contpars.niterate = 10
	contpars.grow = 1.

# Check sky spectra for star and galaxy absorption lines
	if (access (outlist))
	    delete (outlist)
	if (vb)
	    print ("HSKYFILT: Comparing sky to absorption and emission spectra using XCSAO")
	nsky = 0
	nsky0 = 0
	list = inflist
	if (fscan (list, ts1,ts2,ts3,ts4,ts5) != EOF) {
            if (fscan (list, ts1,ts2,ts3,ts4,ts5) != EOF) {
		}
            }
	while (fscan (list, imsky, ap, fib, fibx, fiby, fibr) != EOF) {
	    if (access (skysub))
		delete (skysub)
	    imarith (imsky, "/", skyavg, skysub)
	    pxcsao (skysub, templates=abtemps)
	    round (pxcsao.velocity, "%.3f")
	    vstring = pxcsao.besttemp // " " // round.value
	    rab = pxcsao.r
	    round (rab, "%.3f")
	    vstring = vstring // " " // round.value
	    hedit (imsky, "SKYXCABS", vstring, add+, verify-, show-)

# Check sky spectra for star and galaxy emission lines
	    pxcsao (skysub, templates=emtemps)
	    rem = pxcsao.r
	    round (pxcsao.velocity, "%.3f")
	    vstring = pxcsao.besttemp // " " // round.value
	    round (rem, "%.3f")
	    vstring = vstring // " " // round.value
	    hedit (imsky, "SKYXCEM", vstring, add+, verify-, show-)
	    nsky0 = nsky0 + 1

	    if (rem > rab) {
		round (rem, "%.3f")
		hedit (imsky, "SKYXR", round.value, add+, verify-, show-)
		}
	    else {
		round (rab, "%.3f")
		hedit (imsky, "SKYXR", round.value, add+, verify-, show-)
		}
	    if (vb)
		printf ("%s %.3f %.3f %s\n",imsky,rab,rem, round.value)
		

# If either R value exceeds limit, drop sky from list
	    if (rem > skycert || rab > skycert) {
		if (rem > skycert) {
		    round (rem, "%.3f")
		    vstring = round.value
		    vstring = "Em R " // vstring // " > " // skycert
		    hedit (imsky, "SKYREJ", vstring, add+, verify-, show-)
		    print (imsky," rejected: ",vstring)
		    }
		else if (rab > skycert) {
		    round (rab, "%.3f")
		    vstring = round.value
		    vstring = "Abs R " // vstring // " > " // skycert
		    hedit (imsky, "SKYREJ", vstring, add+, verify-, show-)
		    print (imsky," rejected: ",vstring)
		    }
		else {
		    hedit (imsky, "SKYREJ", delete+, verify-, show-)
		    }
		}
	    else {
		nsky = nsky + 1
		if (nsky == 1) {
		    printf ("%s\n%s\n", tline1, tline2, >outlist)
		    }
		printf ("%s	%3d	%3d	%7.2f	%7.2f	%7.2f\n",imsky, ap,fib,fibx,fiby, fibr, >>outlist)
		}
	    }
	if (nsky < 1) {
	    print ("HSKYFILT: No skies passed contamination test")
	    }
	else {
	    if (vb) {
		if (nsky == nsky0)
		    print ("HSKYFILT: All ",nsky," skies passed the contamination test")
		else
		    print ("HSKYFILT: ",nsky,"/",nsky0," skies passed the contamination test")
		}
	    }

# Recreate sky table with apertures
	if (vb)
	    print ("HSKYFILT: Keeping ",nsky," skies in ",outlist)
#	pwd | scan (ppath)
#	filename (ppath)
#	skydb = filename.filename // "_skylist.db"
	skydb = ddir // "skylist.db"
	if (access (skydb))
	    delete (skydb)
	if (!defpac ("wcstools"))
	    wcstools (motd=no)
	print ("filename	ap	chip	mean	stdev	rabs 	rem 	rhigh", >skydb)
	print ("-----------	---	----	-------	------	-----	-----	-----", >>skydb)
	args = "-tu @" // filelist // " aperture chipnum skymean skystdev"
	args = args // " skyxcabs,3 skyxcem,3 skyxr"
	gethead (args, >>skydb)
	if (dt) {
	    delete (filelist)
	    delete (inflist)
	    delete (skysub)
	    delete (skyavg)
	    }
end
# Aug 30 2007	New task based on Bill Wyatt's skyproc ksh script
# Sep  6 2007	Add maximum R value, SKYXCR, to sky spectrum headers
# Sep  7 2007	Delete SKYREJ header keyword if spectrum passes
# Sep  7 2007	Add CHIPNUM to headers and write database for Susan
# Sep 10 2007	Compute INSKY instead of passing it in
# Sep 11 2007	Add output directory as a parameter
# Sep 12 2007	Make all temporary file names start with "temp"
# Sep 12 2007	Pass fiber radial distance through tables
# Sep 13 2007	Use fixdir so destdir does not need trailing /
