# File hectospec/Scripts/hskylist.cl
# By Doug Mink, after Bill Wyatt's skyproc.ksh
# September 25, 2007

procedure hskylist (spectra)

string spectra="\*.fits"	{prompt="Files to process"}

string skyspectra=""		{prompt="Sky spectra to combine"}
string destdir="skysub/"	{prompt="Output dir for sky-subtracted files"}
bool skysame=yes	{prompt="Use skys only from same chip"}
real fiberlim=306.0	{prompt="Radius limit for non-parked fibers"}
int objchip=0		{prompt="Chip to reduce (1 or 2, 0=both)"}
int skychip=0		{prompt="Chip for sky spectra (1 or 2, 0=both)"}
int nsky=0		{prompt="Number of sky files"}
string skydb		{prompt="Sky filenames and info"}
int nobj=0		{prompt="Number of object files"}
string objdb		{prompt="Object filenames and info"}
int nsky1=0		{prompt="Number of sky spectra from chip 1"}
int nobj1=0		{prompt="Number of object spectra from chip 1"}
string skydb1	{prompt="Chip 1 sky filenames and info"}
string objdb1	{prompt="Chip 1 object filenames and info"}
int nsky2=0		{prompt="Number of sky spectra from chip 2"}
int nobj2=0		{prompt="Number of object spectra from chip 2"}
string skydb2	{prompt="Chip 2 sky filenames and info"}
string objdb2	{prompt="Chip 2 object filenames and info"}
bool deltemp=yes	{prompt="Delete temporary files (yes or no"}
bool verbose=yes	{prompt="Log progress (yes or no)"}
bool debug=no		{prompt="Print debugging information (yes or no)"}

begin
    bool vb, db, dt, keepobj
    real fiber_x, fiber_y, fiber_rad
    file imf, tempilist, tempslist, tempkey, dbroot, tempskey, newkey
    file listfile, ddir
    int aperture, chipnum, fiber, schip, ochip
    int ns, nspec
    string args, imagetype, object, specs, sspec
    string ts1,ts2,ts3,ts4,ts5, tline1, tline2, dline1, dline2
    char tchar

    vb = verbose
    db = debug
    dt = deltemp
    fixdir (destdir)
    ddir = fixdir.newdir
    specs = spectra
    ochip = objchip
    schip = skychip
    sspec = skyspectra

    if (!deftask ("gethead")) {
	wcstools (motd=no)
	}

# Make sure output directory is there
    if (!access (ddir)) {
	mkdir (ddir)
	}

    if (vb) {
	print ("HSKYLIST FiberRadiusLimit = ", fiberlim)
	if (skysame)
	    print ("HSKYLIST:  Skys on same chip")
	else
	    print ("HSKYLIST:  Skys on either chip")
	print ("HSKYLIST: DestDir = ", ddir)
	}

#  Make temporary list of input spectrum files
    tempilist = ddir // mktemp ("tempilist")
    tchar = substr (specs,1,1)
    if (tchar == '@') {
	listfile = substr (specs,2,100)
	copy (listfile, tempilist)
	}
    else {
	if (substr(specs,1,1) == "*") {
	    if (ochip == 1) {
		files ("0*fits", >tempilist)
		files ("10*fits", >>tempilist)
		files ("11*fits", >>tempilist)
		files ("12*fits", >>tempilist)
		files ("13*fits", >>tempilist)
		files ("14*fits", >>tempilist)
		files ("150*fits", >>tempilist)
		}
	    else if (ochip == 2) {
		files ("151*fits", >tempilist)
		files ("152*fits", >>tempilist)
		files ("153*fits", >>tempilist)
		files ("154*fits", >>tempilist)
		files ("155*fits", >>tempilist)
		files ("156*fits", >>tempilist)
		files ("157*fits", >>tempilist)
		files ("158*fits", >>tempilist)
		files ("159*fits", >>tempilist)
		files ("16*fits", >>tempilist)
		files ("17*fits", >>tempilist)
		files ("18*fits", >>tempilist)
		files ("19*fits", >>tempilist)
		files ("2*fits", >>tempilist)
		files ("300*fits", >>tempilist)
		}
	    else
		files (specs, >tempilist)
	    }
	else
	    files (specs, >tempilist)
	}
    newkey = ddir // mktemp ("tempkey")
    if (access (newkey))
	delete (newkey)
    dline1 = "filename     	type	object	ap 	fiber	fiber_x	fiber_y	fiber_r"
    dline2 = "-----------	-------	------	---	-----	-------	-------	-------"
    
# Generate master table of object spectrum keywords or use existing one
    tempkey = ddir // "speckey.db"
    if (!access (tempkey)) {
	if (vb)
	    print ("HSKYLIST: Making master keyword file ",tempkey)
	ns = 0
	print (dline1, >tempkey)
	print (dline2, >>tempkey)
	list = tempilist
	while (fscan (list, imf) != EOF) {
	    ns = ns + 1
	    args = imf // " -n 3 -atu imagetyp object aperture fiber fiber_x fiber_y"
	    gethead (args, >>tempkey)
	    if (db)
		gethead (args)
	    }
	}
    else if (vb)
	print ("HSKYLIST: Using existing master keyword file ", tempkey)

#  Make table(s) of sky/background spectrum files
    dbroot = ddir // "sky"
    if (skysame) {
        skydb1 = dbroot // "_chip1.db"
        skydb2 = dbroot // "_chip2.db"
	if (vb)
	    print ("HSKYLIST: Writing skies to ",skydb1," and ",skydb2)
	if (access (skydb1)) {
	    delete (skydb1)
	    if (db)
		print ("HSKYLIST: Deleted skydb1= ",skydb1)
	    }
	if (access (skydb2)) {
	    delete (skydb2)
	    if (db)
		print ("HSKYLIST: Deleted skydb2= ",skydb2)
	    }
	}
    else {
        skydb = dbroot // "_all.db"
	if (vb)
	    print ("HSKYLIST: Writing skies to ",skydb)
	if (access (skydb)) {
	    delete (skydb)
	    if (db)
		print ("HSKYLIST: Deleted skydb= ",skydb)
	    }
	}

#  Make table(s) of object+ sky/background spectrum files
    dbroot = ddir // "obj"
    if (skysame) {
        objdb1 = dbroot // "_chip1.db"
        objdb2 = dbroot // "_chip2.db"
	if (vb)
	    print ("HSKYLIST: Writing objects to ",objdb1," and ",objdb2)
	if (access (objdb1)) {
	    delete (objdb1)
	    if (db)
		print ("HSKYLIST: Deleted objdb1= ",objdb1)
	    }
	if (access (objdb2)) {
	    delete (objdb2)
	    if (db)
		print ("HSKYLIST: Deleted objdb2= ",objdb2)
	    }
	}
    else {
        objdb = dbroot // "_all.db"
	if (vb)
	    print ("HSKYLIST: Writing objects to ",objdb)
	if (access (objdb)) {
	    delete (objdb)
	    if (db)
		print ("HSKYLIST: Deleted objdb= ",objdb)
	    }
	}

# Edit master keyword table
    nsky = 0
    nsky1 = 0
    nsky2 = 0
    nobj = 0
    nobj1 = 0
    nobj2 = 0
    nspec = 0
    tline1 = ""
    tline2 = ""
    list = tempkey
    if (fscanf (list, "%s", tline1) != EOF) {
	if (fscanf (list, "%s", tline2) != EOF) {
	    }
	}
    tline1 = "filename   	ap 	fiber	fiber_x	fiber_y	fiber_r"
    tline2 = "-----------	---	-----	-------	-------	-------"
    while (fscan (list, imf, imagetype, object, aperture, fiber, fiber_x, fiber_y) != EOF) {
	if (object == "-1") {
	    object = "UNUSED"
	    imagetype = "UNUSED"
	    }
	else if (object == "-2") {
	    object = "BROKEN"
	    imagetype = "BROKEN"
	    }
	else if (object == "0") {
	    object = "SKY"
	    }
	fiber_rad = sqrt (fiber_x * fiber_x + fiber_y * fiber_y)
	round (fiber_rad, "%.2f")
	hedit (imf, "FIBER_R", round.value, add+, verify-, show-)

	nspec = nspec + 1
	if (nspec == 1)
	    printf ("%s\n%s\n",dline1,dline2, >newkey)
	printf ("%s	%s	%s	%3d	%3d	%.2f	%.2f	%.2f\n",
	    imf,imagetype, object, aperture,fiber,fiber_x,fiber_y,fiber_rad,>>newkey)

	if (skysame) {
	    if (aperture <= 150) {
		chipnum = 1
		}
	    else {
		chipnum = 2
		}
	    }
	else {
	    chipnum = 0
	    }

# Check object spectrum list for parked, unused, and broken fibers
	keepobj = no
	if ((imagetype == "object" || imagetype == "OBJECT")) {
	    if (object!="UNUSED" && object!="BROKEN" && object!="PARKED") {
		if (fiber_rad <= fiberlim) {
		    keepobj = yes
		    }
		}
	    }

# Pull all object (including sky) files into a master list
	if (keepobj) {
	    if (skysame) {
		if (chipnum == 1 && ochip < 2) {
		    nobj1 = nobj1 + 1
		    if (nobj1 == 1)
			printf ("%s\n%s\n",tline1,tline2, >objdb1)
		    printf ("%s	%3d	%3d	%.2f	%.2f	%.2f\n",
			    imf,aperture,fiber,fiber_x,fiber_y,fiber_rad,>>objdb1)
		    if (db)
			printf ("HSKYLIST %d: %s added to objdb1\n", nobj1, imf)
		    }
		if (chipnum == 2 && ochip != 1) {
		    nobj2 = nobj2 + 1
		    if (nobj2 == 1)
			printf ("%s\n%s\n",tline1,tline2, >objdb2)
		    printf ("%s	%3d	%3d	%.2f	%.2f	%.2f\n",
			    imf,aperture,fiber,fiber_x,fiber_y,fiber_rad,>>objdb2)
		    if (db)
			printf ("HSKYLIST %d: %s added to objdb2\n", nobj2, imf)
		    }
		}
	    else {
		nobj = nobj + 1
		if (nobj == 1)
		    printf ("%s\n%s\n",tline1,tline2, >objdb)
		printf ("%s	%3d	%3d	%.2f	%.2f	%.2f\n",
			imf,aperture,fiber,fiber_x,fiber_y,fiber_rad,>>objdb)
		if (db)
		    printf ("HSKYLIST %d: %s added to objdb\n",nobj, imf)
		}
	    }

# create a sky file db
	if (sspec == "" && keepobj && (object == "SKY" || object == "sky")) {
	    if (skysame) {
		if (chipnum == 1 && schip < 2) {
		    nsky1 = nsky1 + 1
		    if (nsky1 == 1)
			printf ("%s\n%s\n",tline1,tline2, >skydb1)
		    printf ("%s	%3d	%3d	%.2f	%.2f\n",
			    imf,aperture,fiber,fiber_x,fiber_y,>>skydb1)
		    if (db)
			printf ("HSKYLIST %d: %s added to skydb1\n",nsky1, imf)
		    }
		if (chipnum == 2 && schip != 1) {
		    nsky2 = nsky2 + 1
		    if (nsky2 == 1)
			printf ("%s\n%s\n",tline1,tline2, >skydb2)
		    printf ("%s	%3d	%3d	%.2f	%.2f	%.2f\n",
			    imf,aperture,fiber,fiber_x,fiber_y,fiber_rad,>>skydb2)
		    if (db)
			printf ("HSKYLIST %d: %s added to skydb2\n",nsky2, imf)
		    }
		}
	    else {
	        nsky = nsky + 1
		if (nsky == 1)
		    printf ("%s\n%s\n",tline1,tline2, >skydb)
		printf ("%s	%3d	%3d	%.2f	%.2f	%.2f\n",
			imf,aperture,fiber,fiber_x,fiber_y,fiber_rad,>>skydb)
		if (db)
		    printf ("HSKYLIST %d: %s added to skydb\n", nsky, imf)
		}
	    }
	}

    if (access (newkey)) {
#	type (newkey)
	delete (tempkey)
	rename (newkey, tempkey)
	}

# Generate master table of sky spectrum keywords, if needed
    tempskey = ""
    if (sspec != "") {

#  Make temporary list of input spectrum files
	tempslist = ddir // mktemp ("tempslist")
	tchar = substr (sspec,1,1)
	if (tchar == '@') {
	    listfile = substr (sspec,2,100)
	    copy (listfile, tempslist)
	    }
	else {
	    files (sspec, >tempslist)
	    }

# Generate master table of sky spectrum keywords or use existing one
	tempskey = ddir // "skykey.db"
	if (!access (tempskey)) {
	    if (vb)
		print ("HSKYLIST: Making master keyword file ",tempskey)
	    ns = 0
	    tline1="filename	type	object	ap	fiber	fiber_x	fiber_y"
	    print (tline1, >tempskey)
	    tline2="--------	----	------	--	-----	-------	-------"
	    print (tline2, >>tempskey)
	    list = tempslist
	    while (fscan (list, imf) != EOF) {
		ns = ns + 1
		args = imf // " -n 2 -atu imagetyp object aperture fiber fiber_x fiber_y"
		gethead (args, >>tempskey)
		if (db)
		    gethead (args)
		}
	    }
	else if (vb)
	    print ("HSKYLIST: Using existing sky keyword file ", tempskey)

# Add to sky table from this list
	nspec = 0
	list = tempskey
	if (fscanf (list, "%s", tline1) != EOF) {
	    if (fscanf (list, "%s", tline2) != EOF) {
		}
	    }
	tline1 = "filename   	ap 	fiber	fiber_x	fiber_y	fiber_r"
	tline2 = "-----------	---	-----	-------	-------	-------"
	while (fscan (list, imf, imagetype, object, aperture, fiber, fiber_x, fiber_y) != EOF) {
	    if (object == "-1") {
		object = "UNUSED"
		imagetype = "UNUSED"
		}
	    else if (object == "-2") {
		object = "BROKEN"
		imagetype = "BROKEN"
		}
	    else if (object == "0") {
		object = "SKY"
	    }
	    fiber_rad = sqrt (fiber_x * fiber_x + fiber_y * fiber_y)
	    round (fiber_rad, "%.2f")
	    hedit (imf, "FIBER_R", round.value, add+, verify-, show-)
	    nspec = nspec + 1
	    if (nspec == 1)
		 printf ("%s\n%s\n",dline1,dline2, >newkey)
	    printf ("%s	%s	%s	%3d	%3d	%.2f	%.2f	%.2f\n",
		    imf,imagetype, object, aperture,fiber,fiber_x,fiber_y,fiber_rad,>>newkey)

	    if (skysame) {
		if (aperture <= 150) {
		    chipnum = 1
		    }
		else {
		    chipnum = 2
		    }
		}
	    else {
		chipnum = 0
		}

# Check for broken, parked or unused fibers in the sky list
	    keepobj = no
	    if ((imagetype == "object" || imagetype == "OBJECT")) {
		if (object!="UNUSED" && object!="BROKEN" && object!="PARKED") {
		    if (fiber_rad <= fiberlim) {
			keepobj = yes
			}
		    }
		}

# create a sky file db
	    if (keepobj && (object == "SKY" || object == "sky")) {
		if (debug)
		    print ("HSKYLIST: chipnum=",chipnum," skysame=",skysame," skychip=",schip)
		if (skysame) {
		    if (chipnum == 1 && schip < 2) {
			nsky1 = nsky1 + 1
			if (nsky1 == 1)
			    printf ("%s\n%s\n",tline1,tline2, >skydb1)
			printf ("%s	%3d	%3d	%.2f	%.2f	%.2f\n",
				imf,aperture,fiber,fiber_x,fiber_y,fiber_rad, >>skydb1)
			if (db)
			    printf ("HSKYLIST %d: %s added to skydb1\n", nsky1, imf)
			}
		    if (chipnum == 2 && schip != 1) {
			nsky2 = nsky2 + 1
			if (nsky2 == 1)
			    printf ("%s\n%s\n",tline1,tline2, >skydb2)
			printf ("%s	%3d	%3d	%.2f	%.2f	%.2f\n",
				imf,aperture,fiber,fiber_x,fiber_y,fiber_rad,>>skydb2)
			if (db)
			    printf ("HSKYLIST %d: %s added to skydb2\n", nsky2, imf)
			}
		    }
		else {
	            nsky = nsky + 1
		    if (nsky == 1)
			printf ("%s\n%s\n",tline1,tline2, >skydb)
		    printf ("%s	%3d	%3d	%.2f	%.2f	%.2f\n",
			    imf,aperture,fiber,fiber_x,fiber_y,fiber_rad,>>skydb)
		    if (db)
			printf ("HSKYLIST %d : %s added to skydb\n", nsky, imf)
		    }
		}
	    }
	}

    if (access (newkey)) {
#	type (newkey)
	delete (tempskey)
	rename (newkey, tempskey)
	}

# Return if no sky spectra to subtract
    if (skysame) {
	nsky = nsky1 + nsky2
	nobj = nobj1 + nobj2
	}
    if (vb) {
	if (skysame) {
	    if (nobj <= 0)
		print ("HSKYLIST: No object spectra in this directory.")
	    else
		print ("HSKYLIST: ",nobj1,"+ ",nobj2," object spectra in this directory")
	    if (nsky <= 0)
		print ("HSKYLIST: No sky spectra in this directory.")
	    else
		print ("HSKYLIST: ",nsky1,"+ ",nsky2," sky spectra in this directory")
	    }
	else {
	    if (nobj <= 0)
		print ("HSKYLIST: No object spectra in this directory.")
	    else
		print ("HSKYLIST: ",nobj," object spectra in this directory")
	    if (nsky <= 0)
		print ("HSKYLIST: No sky spectra in this directory.")
	    else
		print ("HSKYLIST: ",nsky," sky spectra in this directory")
	    }
	}

    if (deltemp) {
	if (access (tempilist))
	    delete (tempilist)
	if (access (tempslist))
	    delete (tempslist)
	}
end
# Aug 31 2007	New program
# Sep  4 2007	Add option to use separate list of sky spectra
# Sep  6 2007	Return number of sky spectra on chip1 and chip2 separately
# Sep 11 2007	Add fiberlim as a parameter
# Sep 12 2007	Fix bug in processing objects and skies for chip 2 if only chip 2
# Sep 12 2007	Make all temporary file names start with "temp"
# Sep 12 2007	Do not make sky database from spectrum list if sky list is present
# Sep 12 2007	Copy original input list files to temporary file so not deleted
# Sep 21 2007	Two parameters for object chip (objchip) and sky chip (skychip)
# Sep 25 2007	Add aperture 300 and drop aperture 150 if chip = 2
