# HSPECFIT1 - Subtract rescaled mean spectrum from spectrum
# November 27, 2006
# By Doug Mink, SAO Telescope Data Center

procedure hspecfit1 (objfile, fitfile, outfile)

file	objfile=""	{prompt="Spectrum file"}
file	objap=""	{prompt="Aperture of spectrum to fit"}
file	objdir=""	{prompt="Directory containing object+sky spectrum"}
file	fitfile=""	{prompt="Fit spectrum file"}
file	fitaps=""	{prompt="Range of apertures to average and fit"}
file	fitdir=""	{prompt="Directory containing spectra to fit"}
string	combine="average"	{prompt="Combine fitaps as average or median or sum"}
file	reject="none"	{prompt="Rejection operation for scombine"}
file	outfile=""	{prompt="Output residual spectrum file"}
file	outdir=""	{prompt="Directory for output spectrum"}
real	fitmax=10.0	{prompt="Maximum fit coefficient"}
bool	interact=no	{prompt="Interact with mean spectrum (yes or no)"}
bool	plotres=no	{prompt="Plot result (yes or no)"}
bool	deltemp=yes	{prompt="Delete temporary mean file (yes or no)"}
bool	verbose=no	{prompt="Log progress (yes or no)"}
bool	debug=no	{prompt="Print debugging information (yes or no)"}

begin
	file ims, imin, fname, inspec, outspec, meanfile
	bool vb

	inspec = objfile
	outspec = outfile
	vb = verbose

#  Combine spectra to be fit to object spectrum
	meanfile = mktemp ("specfit") // ".fits"
	scombine.input = fitfile
	scombine.output = meanfile
	scombine.noutput = ""
	scombine.logfile = "STDOUT"
	scombine.apertures = fitaps
	scombine.group = "all"
	scombine.combine = combine
	scombine.reject = reject
	scombine.first = yes
	scombine.w1 = INDEF
	scombine.w2 = INDEF
	scombine.dw = INDEF
	scombine.nw = INDEF
	scombine.log = no
	scombine.scale = "none"
	scombine.zero = "none"
	scombine.weight = "none"
	scombine.sample = ""
	scombine.lthreshold = 0.000001
	scombine.hthreshold = INDEF
	scombine.nlow = 1
	scombine.nhigh = 1
	scombine.nkeep = 1
	scombine.mclip = no
	scombine.lsigma = 3.
	scombine.hsigma = 3.
	scombine.rdnoise = "0."
	scombine.gain = "1."
	scombine.snoise = "0."
	scombine.sigscale = 0.1
	scombine.pclip = -0.5
	scombine.grow = 0
	scombine.blank = 0.
	scombine (fitfile, meanfile)

#  Set up parameters to fit combined spectrum to object spectrum

	if (!defpac ("svdfit")) {
	    svdfit
	    }

#  Continuum removal
	contres.c_interactive = no
	contres.c_sample = "*"
	contres.c_function = "spline3"
	contres.naverage = 1
	contres.order = 15
	contres.low_reject = 2.
	contres.high_reject = 2.
	contres.niterate = 10
	contres.grow = 1.

#  Spectrum fitting and removal
	svdres.spectra = inspec
	svdres.specnum = objap
	svdres.specband = 1
	svdres.specdir = objdir
	svdres.eigenfile = meanfile
	svdres.eigenum = "1"
	svdres.eigenband = 1
	svdres.eigendir = fitdir
	svdres.eigentrack = no
	svdres.resfile = outspec
	svdres.resdir = outdir
	svdres.res_format = "multispec"
	svdres.st_lambda = INDEF
	svdres.end_lambda = INDEF
	svdres.nbin = INDEF
	svdres.interp_mode = "spline3"
	svdres.obj_plot = no
	svdres.vec_plot = no
	svdres.res_plot = plotres
	svdres.plot_int = no
	svdres.fixbad = "fill"
	svdres.badlines = "badlines.dat"
	svdres.cont_remove = "subtract"
	svdres.vec_cont = "subtract"
	svdres.cont_add = no
	svdres.fitmax = fitmax
	svdres.logfiles = "STDOUT,svdres.log"
	svdres.nsmooth = 0
	svdres.ablines = "ablines.dat"
	svdres.emlines = "emlines.dat"
	svdres.linedir = "svdfit$lib/"
	svdres.dispmode = 1
	svdres.displot = yes
	svdres.device = "stdgraph"
	svdres.curmode = no
	svdres.contsub_plot = no
	svdres.verbose = vb
	svdres.debug = debug
	svdres.nsum = 1

#	dpar ("svdres",>"svdres.par")
	svdres (inspec,outspec)

	if (deltemp) {
	    delete (meanfile)
	    }

	if (vb) {
	    print (inspec," local mean spectrum removed using SVDRES")
	    }
end

# Nov 27 2006	New task based on skyfit1.cl
