#!/data/mmti/bin/ksh

# hsplit: Make 1D spectra from Hectospec multispec file
# By Doug Mink, October 31, 2003

integer nfiles
typeset token
keyfile="/data/IRAF2.12.2/iraf.pkg/hectospec/keywords"

if test -z $1
    then
    echo "hsplit: Make 1D spectra (fiber.object) from Hectospec multispec file"
    echo "        hsplit [-v] file_name(without .ms.*)"
    echo "        -v: list output files as they are made"
    echo "        -k: use this file for list of keywords to delete"
    exit
    fi
v=
idfile=
wcsdir=/data/oir/wcs/bin.solaris

# Check for verbose flag as first argument
if test $1 = "-v"
    then
    #echo verbose mode
    v=true
    shift
fi
if test $1 = "-k"
    then
    keyfile=$2
    if test $v
	then
	echo Delete keywords listed in file $idfile
	fi
    shift
    shift
fi
if test $1 = "-v"
    then
    #echo verbose mode
    v=true
    if test $idfile
	then
	echo Object name and position from $idfile
	fi
    shift
fi


# Make separate file to pre-edit header for output files
tempfile=$1t.ms.fits

# Copy .fits or .imh output from DOFIBERS into file to be edited
msfile=$1.ms.fits
if test -r $msfile
    then
    cp $msfile $tempfile
else
    msfile=$1.ms.fit
    if test -r $msfile
	then
	cp $msfile $tempfile
    else
	msfile=$1.ms.imh
	if test -r $msfile
	    then
	    $wcsdir/i2f -o $tempfile $msfile
	else
	    echo "*** HSPLIT cannot read input file $1.ms.imh or $1.ms.fits"
	    exit 1
	fi
    fi
fi

# Get number of spectra in multispec file
nfiles=`$wcsdir/gethead $msfile naxis2`

# Delete unneeded keywords from header (mostly APNUMnn)
if test $v
    then
    echo delhead $tempfile @$keyfile
    fi
$wcsdir/delhead $tempfile @$keyfile
#$wcsdir/keyhead $tempfile utmiddle=utmid

# Change header position keywords
tra=`$wcsdir/gethead $tempfile ra`
tra=${tra/: /:0}
tra=${tra/: /:0}
tdec=`$wcsdir/gethead $tempfile dec`
tdec=${tdec/: /:0}
tdec=${tdec/: /:0}
tepoch=`$wcsdir/gethead $tempfile equinox`
tepoch=`$wcsdir/gethead $tempfile epoch`
field=`$wcsdir/gethead $tempfile object`
field=${field/ /_}
field=${field/ /_}
skykey="field="$field" type=0 tra="$tra" tdec="$tdec" tepoch="$tepoch
if test $v
    then
	echo $skykey
    fi

# Set place-holding keywords in temporary file header
fpkey="epoch=2000.0 equinox=2000.0 wcsdim=1 imagetyp=OBJECT"
apkey="fiber=0 aprange=0:0 fiber_x=0 fiber_y=0 catname=name catnum=0 aperture=0"
#echo $wcsdir/sethead $tempfile $fpkey $skykey $apkey
$wcsdir/sethead $tempfile $fpkey $skykey $apkey

# If 1d directory does not exist, create it
dir1d=1d.$1
m=true
if test -d $dir1d
    then
    m=""
fi

if test $m
    then
    if test $v
	then
	echo making directory $dir1d
	fi
    mkdir $dir1d
    fi

# Extract spectra one at a time
for ((i=1; i <= nfiles; i++))
    do
    ispec=`printf "%d" i`

# Get APNUM keyword value for this spectrum and extract fiber and x range
    apkw=apnum$ispec
    apval=`$wcsdir/gethead -u $msfile $apkw`
    integer j=0
    for n in $apval
	do token[j]="$n"
	    j=j+1
	done
    aperture=${token[0]}
    type=${token[1]}
    x1=${token[2]}
    x2=${token[3]}
    range=$x1:$x2
    apkey=`echo specnum=$i aperture=$aperture aprange=$range`

# Get pointing direction from APID keyword
    apkw=apid$ispec
    apval=`$wcsdir/gethead $msfile $apkw`
    j=0
    for n in $apval
	do token[j]="$n"
	    j=j+1
	done
    name=${token[0]}
    ra=${token[1]}
    dec=${token[2]}
    catnum=${token[3]}
    fiber=${token[4]}
    x=${token[5]}
    y=${token[6]}
    if test $name = "target"
	then
	objkey=`echo object=$field\_$catnum catobj=target`
    else
	objkey=`echo object=$name catobj=$name`
    fi
    skykey=`echo type=$type ra=$ra dec="$dec catnum=$catnum`
    fibkey=`echo fiber=$fiber fiber_x=$x fiber_y=$y`
    if test $v
	then
	    echo $objkey $apkey
	    echo $skykey
	    echo $fibkey
	fi

# Extract this spectrum into a separate FITS file
    outfile=`$wcsdir/imextract -n $tempfile $ispec $objkey $skykey $apkey $fibkey`

# Make up new file name from fiber number and object name
    apname=`printf "%03d" $aperture`
    if test $name = "target"
	then
	newfile=`echo $dir1d/$apname.$field\_$catnum.fits`
    else
	newfile=`echo $dir1d/$apname.$name\_f$fiber.fits`
    fi
    mv $outfile $newfile
    if test $v
	then
	echo mv $outfile $newfile
 	echo
	fi
    done
/bin/rm $tempfile

# Feb 22 2000	New program based on split2df
# May 25 2000	Add code to deal with possible input FITS file

# Oct  9 2003	Add -k option to specify keyword deletion list file
# Oct 28 2003	Update parsing code and rewrite to work on actual data
# Oct 31 2003	Save name from APIDn keyword value; use for filename if not "target"
