# Run the HectoSpec early processing steps up to tracing
#
# Bill Wyatt, 2003-11-06 et. seq.
#

# requires starbase and foreign task 'gethead' to be in place

# Still to do - make inbetween paths if need be for procdir
# distinguish comparison and flat groups so as to combine only those
# taken at hearby times, i.e. not a morning set and evening set.


procedure hspreproc (input)

# ccdproc params
string input  = "*.fits" {prompt="input file(s)"}

bool overscan = yes  {prompt="switch for ccdproc"}
bool trim     = yes  {prompt="switch for ccdproc"}
bool zerocor  = no   {prompt="switch for ccdproc"}
bool darkcor  = no   {prompt="switch for ccdproc"}
bool fixpix   = yes  {prompt="switch for bad pixel correction"}

file fixfile  = "BPM" {prompt="bad pixel mask file"}

real darkexp  =  60.0  {prompt="min darktime for darks, in secs"}
real flatexp  =   3.0  {prompt="min exptime for flats, in secs"}
real objexp   =  60.0  {prompt="min exptime for objects, in secs"}
real compexp  = 300.0  {prompt="min exptime for comps, in secs"}

string procdir = "" {prompt="Output dir for processed images"}
string compnames= "HEAR|TH[-_ ]*AR|HENEAR|IRNE" {prompt="Possible comp OBJECT names"}

bool verbose = yes  {prompt="Verbose output"}

begin

   string  inlist
   string  procid

   file  objlist
   file  complist
   file  sflatlist
   file  dflatlist
   file  zerolist
   file  darklist
   file  tmplist
   file  tmplist2
   file  tmplist3
   string  xprocdir
   string  objn
   string  dbname
   string  stuff
   int flen

   inlist = input
   xprocdir = procdir

   procid = mktemp("hsp_")   # really the procid plus a character
   procid = osfn("tmp$") // procid

   dbname = "keyval.db"   

# make the procdir if it has not been input
   if(strlen(xprocdir) == 0) {
      path | scan(xprocdir)
      basenm(xprocdir) | scan(xprocdir)
      xprocdir = "../procdir/" // xprocdir // "/"
   }      
   if(access(xprocdir) == no) { pmkdir(xprocdir) }
   if(access(xprocdir) == no) { 
     printf("Error in hspreproc - can't make procdir = %s?\n", xprocdir)
     bye
   } else if(verbose) {
     printf("\nProcessed output to %s\n", xprocdir);
   }

   if(access(dbname)) {
     if(verbose) { printf("\n%s exists - not recreate
d\n",dbname) }
   } else {
     gethead("-tha",inlist,"imagetyp","object","exptime","darktime") | 
   compute('gsub("[ /#]", "_", OBJECT); ')  |
   compute('if(toupper(OBJECT) ~ /(' // compnames // ')/) {IMAGETYP = "comp"}') |
   compute('if(toupper(OBJECT) ~ /(TWILITE|DAWN|DUSK|TWILIGHT)/) {OBJECT ="skyflat"}') |
   compute('if(toupper(OBJECT) ~ /FLAT/) { IMAGETYP = "flat"}')  |
     sorttable("-o", dbname, "FILENAME")
   }   

   objlist  = "objfiles.db"
   zerolist = "prezeros"
   darklist = "predarks"
   complist = "precomps"
   dflatlist = "predomeflats"
   sflatlist = "preskyflats"

   tmplist  = procid // "objfiles"
   tmplist2 = procid // "objfiles2"
   tmplist3 = procid // "objfiles3"

# Split out the (currently not used) files for BIAS and DARK corrections
   row("-i", dbname, 
       'toupper(IMAGETYP) ~ "DARK" && DARKTIME >= ' // darkexp ) |
   headoff("-o", darklist, "FILENAME")

   row("-i", dbname, 
      'toupper(IMAGETYP) ~ /(ZERO|BIAS)/')   |
   headoff("-o", zerolist, "FILENAME")

# remove them from the object list
   row("-i", dbname, "-o", objlist,
     'toupper(IMAGETYP) !~ /ZERO|BIAS|DARK/  && ' //
     ' (toupper(IMAGETYP) ~ "FLAT"   && EXPTIME >= ' // flatexp // ' ||' //
     '  toupper(IMAGETYP) ~ "OBJECT" && EXPTIME >= ' // objexp  // ' ||' //
     '  toupper(IMAGETYP) ~ "COMP"   && EXPTIME >= ' // compexp // ')' )

# split out for possible future use the zero, bias, flat & comparison lists
   if(zerocor) {
     flen = 0
     row("-i", dbname, 'toupper(IMAGETYP) ~ /(ZERO|BIAS)/')   |
     headoff("-o", zerolist, "FILENAME")
     if(access(zerolist) == no) {
         zerocor = no
     } else {
         count(zerolist) | scan(flen,stuff)
         if(flen == 0) { zerocor = no; delete(zerolist); }
     }
     if(verbose && access(zerolist) == no) { printf("No bias files found\n") }
   }
   if(darkcor) {
     flen = 0
     row("-i", dbname,
       'toupper(IMAGETYP) ~ "DARK" && DARKTIME >= ' // darkexp ) |
     headoff("-o", darklist, "FILENAME")
     if(access(darklist) == no) {
         darkcor = no
     } else {
         count(darklist) | scan(flen,stuff)
         if(flen == 0) { darkcor = no; delete(darklist); 
         } else {
	    printf("\nDark file statistics:\n")
            imstatistics(@darklist)
         }
     }
     if(verbose && access(darklist) == no) { printf("No dark files found\n") }
   }

   row("-i", objlist, 'toupper(IMAGETYP) ~ "COMP"')   |
   headoff("-o", complist, "FILENAME")

   row("-i", objlist, 
       'toupper(IMAGETYP) ~ /FLAT/ && toupper(OBJECT) ~ /DOME[ _]*FLAT/')   |
   headoff("-o", dflatlist, "FILENAME")

   row("-i", objlist, 
       'toupper(IMAGETYP) ~ /FLAT/ && toupper(OBJECT) ~ /SKY[ _]*FLAT/')   |
   headoff("-o", sflatlist, "FILENAME")

# lists now are made - ccdproc them.
   ccdproc.output   = ""
   ccdproc.ccdtype  = ""
   ccdproc.fixpix   = fixpix
   ccdproc.fixfile  = fixfile
   ccdproc.flatcor  = no       # HARDWIRED
   ccdproc.sflatcor = no       # HARDWIRED
   ccdproc.output   = ""       # HARDWIRED
   ccdproc.biassec  = "!BIASSEC"
   ccdproc.trimsec  = "!TRIMSEC"
   ccdproc.overscan = overscan
   ccdproc.trim     = trim

# proc the bias, if zerocor=yes
   ccdproc.zerocor  = no
   ccdproc.darkcor  = no

   cl < "hectopkg$combine.dan.par"

   if(access("zero.fits") && zerocor && verbose) {
     printf("\nzero.fits exists, skipping\n")
   } else if(zerocor) { 
      if(verbose) { 
        printf("\nProcess and combine zeros:\n")
        row("-i", dbname,
           'toupper(IMAGETYP) ~ "ZERO"') |
        justify("FILENAME", "OBJECT", "IMAGETYP", "DARKTIME")
      }
      ccdproc("@" // zerolist) 
      combine("@" // zerolist, "zero", combine="average", rejmasks="zero_rej")
      printf("\nBias file statistics:\n")
      imstatistics(@zerolist)
      printf("\nCombined bias file  statistics:\n")
      imstatistics("zero.fits")
      if(verbose) { printf("Done.\n") }
   }
   if(zerocor) { 
      ccdproc.zero = "zero" 
      ccdproc.zerocor=yes 
   }
   if(access("dark.fits") && zerocor && verbose) {
     printf("\ndark.fits exists, skipping\n")
   } else if(darkcor) { 
      if(verbose) { 
        printf("\nProcess and combine darks:\n")
        row("-i", dbname,
           'toupper(IMAGETYP) ~ "DARK" && DARKTIME >= ' // darkexp ) |  
        justify("FILENAME", "OBJECT", "IMAGETYP", "DARKTIME")
      }
      ccdproc("@" // darklist) 
      combine("@" // darklist, "dark", rejmasks="dark_rej",
                combine="average", weight="!DARKTIME")
      printf("\nDark file statistics:\n")
      imstatistics(@darklist)
      printf("\nCombined dark file  statistics:\n")
      imstatistics("dark.fits")
      if(verbose) { printf("Done.\n") }
   }
   if(darkcor) { 
      ccdproc.dark = "dark" 
      ccdproc.darkcor=yes 
   }

   if(verbose) {
     printf("\nCalling ccdproc on:\n")
     justify("-i", objlist, "FILENAME", "OBJECT", "IMAGETYP", "EXPTIME")
    printf("\n")
   }
   headoff("-i", objlist, "-o", tmplist, "FILENAME")

   ccdproc("@" // tmplist,  merge=no)
   

# Combine various file sets
   if(access(tmplist3)) {  delete(tmplist3) }

   cl < "hectopkg$combine.dan.par"

   if(access("comp.fits")) {
     if(verbose) { printf("\ncomp.fits exists, skipping\n") }
     print("comp", >>tmplist3)
   } else if(access(complist)) {
     if(verbose) { printf("\nCombining comps:\n") }
     combine("@" // complist, "comp", rejmasks="comp_rej")
     if(access("comp.fits")) { 
       printf("%s\n", "comp", >>tmplist3) }
   }
   if(access("skyflat.fits")) {
     if(verbose) { printf("\nskyflat.fits exists, skipping\n") }
     print("skyflat", >>tmplist3)
   } else if(access(sflatlist)) {
     if(verbose) { printf("\nCombining sky flats:\n") }
     combine("@" // sflatlist, "skyflat", rejmasks="skyflat_rej")
     if(access("skyflat.fits")) { print("skyflat", >>tmplist3) }
   }
   if(access("domeflat.fits")) {
     if(verbose) { printf("\ndomeflat.fits exists, skipping\n") }
     print("domeflat", >>tmplist3)
   } else if(access(dflatlist)) {
     if(verbose) { printf("\nCombining dome flats:\n") }
     combine("@" // dflatlist, "domeflat", rejmasks="domeflat_reg")
     if(access("domeflat.fits")) { print("domeflat", >>tmplist3) }
   }

# detect separate objects to combine
   row("-i", objlist, 'toupper(IMAGETYP) ~ "OBJECT"') |
   sorttable("-u","OBJECT")                           |
   headoff("-o", tmplist, "OBJECT")

   list = tmplist
   while(fscan(list, objn) != EOF) {
     row("-i", objlist, 'OBJECT ~ /' // objn // '/')  |
     headoff("-o", tmplist2, "FILENAME")

     if(access(objn) || access(objn // ".fits")) {
        printf("\n Object %s already combined, skipping\n", objn)
     } else {
        printf("\n Combining object %s\n",objn)
        if(verbose) { type(tmplist2) }
        combine("@" // tmplist2, objn, rejmasks=objn // "_reg")
     }
     printf("%s\n", objn, >>tmplist3)
   }

# Gaincorrect the file sets
   if(verbose) { 
     printf("\nGain correction on:\n")
     type(tmplist3)
   }
   gaincorr("@" // tmplist3, "", vb=no)

# combine into single image in the procdir
   if(verbose) {
     printf("\nCombining into one image in %s\n", xprocdir)
   }
   list = tmplist3
   while(fscan(list, objn) != EOF) {
     if(verbose) { printf("Joining %s...\n",objn) }
     if(access(xprocdir // objn // ".fits")) { 
        delete(xprocdir // objn // ".fits") 
     } else if (access(xprocdir // objn)) {
        delete(xprocdir // objn)
     }
     imjoin(objn // "[1][*,*]," // objn // "[2][-*,*]," // objn // "[3][*,*]," // objn // "[4][-*,*]",
            xprocdir // objn,
            1,
            verbose=verbose)
   }
   if(verbose) { print("\nAll done.") }

   delete(procid // "*")
#
end

# 2003-11-10 Bill Wyatt - first working version.
# 2003-11-14 Bill Wyatt - added bias and dark processing
